@@ -1,5 +1,5 @@
 /*
- * Copyright 2003 Chris Morgan <cmorgan@alum.wpi.edu>
+ * Copyright 2003-2004 Chris Morgan <cmorgan@alum.wpi.edu>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -134,7 +134,7 @@ typedef struct jack_driver_s
 
   unsigned int     volume[MAX_OUTPUT_PORTS];      /* percentage of sample value to preserve, 100 would be no attenuation */
   enum JACK_VOLUME_TYPE volumeEffectType;         /* linear or dbAttenuation, if dbAttenuation volume is the number of dBs of
-                                                     attenuation to apply, 0 volume being no attenation, full volume */
+                                                     attenuation to apply, 0 volume being no attenuation, full volume */
 
   long             position_byte_offset;          /* an offset that we will apply to returned position queries to achieve */
                                                   /* the position that the user of the driver desires set */
@@ -151,6 +151,10 @@ typedef struct jack_driver_s
 } jack_driver_t;
 
 
+char *client_name; /* the name bio2jack will use when creating a new
+                             jack client. client_name_%deviceID% will be used */
+
+
 /* enable/disable code that allows us to close a device without actually closing the jack device */
 /* this works around the issue where jack doesn't always close devices by the time the close function call returns */
 #define JACK_CLOSE_HACK 1
@@ -181,8 +185,6 @@ static void JACK_ResetFromDriver(jack_driver_t *drv);
 static long JACK_GetPositionFromDriver(jack_driver_t *drv, enum pos_enum position, int type);
 
 
-
-
 /* Return the difference between two timeval structures in terms of milliseconds */
 long TimeValDifference(struct timeval *start, struct timeval *end)
 {
@@ -343,7 +345,7 @@ void sample_silence_dS (sample_t *dst, unsigned long nsamples)
 /******************************************************************
  *    JACK_callback
  *
- * everytime the jack server wants something from us it calls this
+ * every time the jack server wants something from us it calls this
  * function, so we either deliver it some sound to play or deliver it nothing
  * to play
  */
@@ -353,6 +355,8 @@ static int JACK_callback (nframes_t nframes, void *arg)
   jack_driver_t* drv = (jack_driver_t*)arg;
   unsigned int i;
 
+  gettimeofday(&drv->previousTime, 0); /* record the current time */
+
 #if CALLBACK_TRACE
   TRACE("nframes %ld, sizeof(sample_t) == %d\n", (long)nframes, sizeof(sample_t));
 #endif
@@ -536,7 +540,6 @@ static int JACK_callback (nframes_t nframes, void *arg)
     } /* while(jackFramesAvailable && drv->pPlayPtr) */
 
 
-    gettimeofday(&drv->previousTime, 0); /* record the current time */
     drv->written_client_bytes+=read;
     drv->played_client_bytes+=drv->clientBytesInJack; /* move forward by the previous bytes we wrote since those must have finished by now */
     drv->clientBytesInJack = read; /* record the input bytes we wrote to jack */
@@ -590,7 +593,6 @@ static int JACK_callback (nframes_t nframes, void *arg)
 #if CALLBACK_TRACE
       TRACE("PAUSED or STOPPED or CLOSED, outputting silence\n");
 #endif
-      gettimeofday(&drv->previousTime, 0); /* record the current time */
 
       /* output silence if nothing is being outputted */
       for(i = 0; i < drv->num_output_channels; i++)
@@ -753,8 +755,8 @@ static bool JACK_SendMessage(jack_driver_t* this, enum cmd_enum command, long da
 static int JACK_OpenDevice(jack_driver_t* drv)
 {
   const char** ports;
+  char *our_client_name = 0;
   unsigned int i;
-  char client_name[64];
   int failed = 0;
 
   TRACE("creating jack client and setting up callbacks\n");
@@ -781,20 +783,28 @@ static int JACK_OpenDevice(jack_driver_t* drv)
   /* set up an error handler */
   jack_set_error_function(JACK_Error);
 
+
+  /* build the client name */
+  our_client_name = (char*)malloc(snprintf(our_client_name, 0, "%s_%02d",
+                                           client_name, drv->deviceID) + 1);
+  sprintf(our_client_name, "%s_%02d", client_name, drv->deviceID);
+
   /* try to become a client of the JACK server */
-  snprintf(client_name, sizeof(client_name), "bio2jack_%d_%d", 0, getpid());
-  TRACE("client name '%s'\n", client_name);
-  if ((drv->client = jack_client_new(client_name)) == 0)
+  TRACE("client name '%s'\n", our_client_name);
+  if ((drv->client = jack_client_new(our_client_name)) == 0)
   {
     /* try once more */
     TRACE("trying once more to jack_client_new");
-    if ((drv->client = jack_client_new(client_name)) == 0)
+    if ((drv->client = jack_client_new(our_client_name)) == 0)
     {
       ERR("jack server not running?\n");
+      free(our_client_name);
       return ERR_OPENING_JACK;
     }
   }
 
+  free(our_client_name);
+
   TRACE("setting up jack callbacks\n");
 
   /* JACK server to call `JACK_callback()' whenever
@@ -864,13 +874,21 @@ static int JACK_OpenDevice(jack_driver_t* drv)
     }
 
     /* display a trace of the output ports we found */
-    for(i = 0; ports[i]; i++)
-	TRACE("ports[%d] = '%s'\n", i, ports[i]);
+    if(ports)
+    {
+        for(i = 0; ports[i]; i++)
+            TRACE("ports[%d] = '%s'\n", i, ports[i]);
+    }
 
-    /* see if we have enough ports */
-    if(i < drv->num_output_channels)
+    /* ensure that we found enough ports */
+    if(!ports || (i < drv->num_output_channels))
     {
       TRACE("ERR: jack_get_ports() failed to find ports with jack port flags of 0x%lX'\n", drv->jack_port_flags);
+#if JACK_CLOSE_HACK
+      JACK_CloseDevice(drv, TRUE);
+#else
+      JACK_CloseDevice(drv);
+#endif
       return ERR_PORT_NOT_FOUND;
     }
 
@@ -893,7 +911,6 @@ static int JACK_OpenDevice(jack_driver_t* drv)
     {
       TRACE("jack_get_ports() portname %d of '%s\n", i, drv->jack_port_name[i]);
       ports = jack_get_ports(drv->client, drv->jack_port_name[i], NULL, drv->jack_port_flags);
-      TRACE("ports[%d] = '%s'\n", 0, ports[0]);      /* display a trace of the output port we found */
 
       if(!ports)
       {
@@ -901,6 +918,8 @@ static int JACK_OpenDevice(jack_driver_t* drv)
           return ERR_PORT_NOT_FOUND;
       }
 
+      TRACE("ports[%d] = '%s'\n", 0, ports[0]);      /* display a trace of the output port we found */
+
       /* connect the port */
       TRACE("jack_connect() to port %d('%p')\n", i, drv->output_port[i]);
       if(jack_connect(drv->client, jack_port_name(drv->output_port[i]), ports[0]))
@@ -916,7 +935,11 @@ static int JACK_OpenDevice(jack_driver_t* drv)
   if(failed)
   {
     TRACE("failed, closing and returning error\n");
+#if JACK_CLOSE_HACK
     JACK_CloseDevice(drv, TRUE);
+#else
+    JACK_CloseDevice(drv);
+#endif
     return ERR_OPENING_JACK;
   }
 
@@ -950,10 +973,10 @@ static void JACK_CloseDevice(jack_driver_t* drv)
     TRACE("closing the jack client thread\n");
     if(drv->client)
     {
-      //FIXME: deactivate or no, jack_client_close() is blocking...
-      //jack_deactivate(drv->client); /* supposed to help the jack_client_close() to succeed */
       TRACE("after jack_deactivate()\n");
-      jack_client_close(drv->client);
+      int errorCode = jack_client_close(drv->client);
+      if(errorCode)
+          ERR("jack_client_close() failed returning an error code of %d\n", errorCode);
     }
 
     JACK_ResetFromDriver(drv);
@@ -1034,7 +1057,7 @@ int JACK_Open(int* deviceID, unsigned int bits_per_channel, unsigned long *rate,
  * see JACK_Open() for comments
  * NOTE: jack_port_name has three ways of being used:
  *       - NULL - finds all ports with the given flags
- *       - A single regex string used to retreve all port names
+ *       - A single regex string used to retrieve all port names
  *       - A series of port names, one for each output channel
  *
  * we set *deviceID
@@ -1132,7 +1155,11 @@ int JACK_OpenEx(int* deviceID, unsigned int bits_per_channel,
     TRACE("rate of %ld doesn't match jack sample rate of %ld, returning error\n",
 	  *rate, drv->sample_rate);
     *rate = drv->sample_rate;
+#if JACK_CLOSE_HACK
     JACK_CloseDevice(drv, TRUE);
+#else
+    JACK_CloseDevice(drv);
+#endif
     releaseDriver(drv);
     return ERR_RATE_MISMATCH;
   }
@@ -1698,7 +1725,7 @@ long JACK_GetSampleRate(int deviceID)
   return return_val;
 }
 
-/* Initialize the jack porting libarary to a clean state */
+/* Initialize the jack porting library to a clean state */
 void JACK_Init(void)
 {
   jack_driver_t *drv;
@@ -1732,6 +1759,9 @@ void JACK_Init(void)
     pthread_mutex_init(&drv->mutex, NULL);
   }
 
+  client_name = 0; /* initialize the name to null */
+  JACK_SetClientName("bio2jack");
+
   TRACE("finished\n");
 }
 
@@ -1759,6 +1789,26 @@ unsigned long JACK_GetJackBufferedBytes(int deviceID)
   return return_val;
 }
 
+/* set the client name that will be reported to jack when we open a */
+/* connection via JACK_OpenDevice() */
+void JACK_SetClientName(char *name)
+{
+  if(name)
+  {
+    if(client_name) free(client_name);
+
+    /* jack_client_name_size() is the max length of a client name, including
+       the terminating null. */
+    int size = strlen(name) + 1; /* take into account the terminating null */
+    if(size > jack_client_name_size()) size = jack_client_name_size();
+
+    client_name = malloc(size);
+    if(client_name)
+        snprintf(client_name, size, "%s", name);
+    else
+        ERR("unable to allocate %d bytes for client_name\n", size);
+  }
+}
 
 /* unit testing code */
 #if 0
