@@ -76,9 +76,7 @@ void jack_destructor()
   TRACE("cleanup\n");
 
   if((errval = JACK_Close(driver)))
-  {
     ERR("error closing device, errval of %d\n", errval);
-  }
 
   /* only clean this up if we have the function to call */
   if(isXmmsFrequencyAvailable)
@@ -91,6 +89,7 @@ void jack_destructor()
 }
 #endif /* #if HAVE_CTOR */
 
+
 void jack_sample_rate_error(void)
 {
 	dialog = gtk_dialog_new();
@@ -119,6 +118,7 @@ void jack_sample_rate_error(void)
 	gtk_widget_grab_focus(button);
 }
 
+
 /* Return the number of milliseconds of audio data that has been */
 /* written out to the device */
 gint jack_get_written_time(void)
@@ -130,6 +130,7 @@ gint jack_get_written_time(void)
   return return_val;
 }
 
+
 /* Return the current number of milliseconds of audio data that has */
 /* been played out of the audio device, not including the buffer */
 gint jack_get_output_time(void)
@@ -146,6 +147,7 @@ gint jack_get_output_time(void)
   return return_val;
 }
 
+
 /* returns TRUE if we are currently playing */
 /* NOTE: this was confusing at first BUT, if the device is open and there */
 /* is no more audio to be played, then the device is NOT PLAYING */
@@ -191,6 +193,10 @@ void jack_init(void)
   TRACE("initializing\n");
   JACK_Init(); /* initialize the driver */
 
+  /* set the bio2jack name so users will see xmms-jack in their */
+  /* jack client list */
+  JACK_SetClientName("xmms-jack");
+
   xmmslibhandle = dlopen("libxmms.so", RTLD_NOW);
   if(xmmslibhandle)
   {
@@ -243,6 +249,7 @@ void jack_init(void)
 #endif /* #if HAVE_CTOR */
 }
 
+
 /* Return the amount of data that can be written to the device */
 gint jack_free(void)
 {
@@ -264,6 +271,7 @@ gint jack_free(void)
   return return_val;
 }
 
+
 /* Close the device */
 void jack_close(void)
 {
@@ -285,6 +293,7 @@ void jack_close(void)
 #endif /* #if HAVE_CTOR */
 }
 
+
 /* Open the device up */
 gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
 {
@@ -500,14 +509,14 @@ void jack_get_volume(int *l, int *r)
 #endif
 }
 
+
 void jack_about(void)
 {
-
 	dialog = gtk_dialog_new();
-	gtk_window_set_title(GTK_WINDOW(dialog), ("About jack Driver 0.11"));
+	gtk_window_set_title(GTK_WINDOW(dialog), ("About jack Driver 0.12"));
 	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
 	label = gtk_label_new((
-		"XMMS jack Driver 0.11\n\n "
+		"XMMS jack Driver 0.12\n\n "
 		"xmms-jack.sf.net\n"
 		"Chris Morgan<cmorgan@alum.wpi.edu>\n"));
 
@@ -523,6 +532,7 @@ void jack_about(void)
 	gtk_widget_grab_focus(button);
 }
 
+
 OutputPlugin jack_op =
 {
 	NULL,
@@ -544,6 +554,7 @@ OutputPlugin jack_op =
 	jack_get_written_time,
 };
 
+
 OutputPlugin *get_oplugin_info(void)
 {
 	return &jack_op;
